@@ -363,6 +363,7 @@ QgsRasterLayerRenderer::QgsRasterLayerRenderer( QgsRasterLayer *layer, QgsRender
   {
     switch ( temporalProperties->mode() )
     {
+      case Qgis::RasterTemporalMode::FixedDateTime:
       case Qgis::RasterTemporalMode::FixedTemporalRange:
       case Qgis::RasterTemporalMode::RedrawLayerOnly:
       case Qgis::RasterTemporalMode::FixedRangePerBand:
@@ -640,9 +641,21 @@ bool QgsRasterLayerRenderer::forceRasterRender() const
 
   if ( QgsRasterRenderer *renderer = mPipe->renderer() )
   {
-    if ( !( renderer->flags() & Qgis::RasterRendererFlag::InternalLayerOpacityHandling )
-         && renderContext()->testFlag( Qgis::RenderContextFlag::UseAdvancedEffects ) && ( !qgsDoubleNear( mLayerOpacity, 1.0 ) ) )
-      return true;
+    if ( !( renderer->flags() & Qgis::RasterRendererFlag::InternalLayerOpacityHandling ) )
+    {
+      switch ( renderContext()->rasterizedRenderingPolicy() )
+      {
+        case Qgis::RasterizedRenderingPolicy::Default:
+        case Qgis::RasterizedRenderingPolicy::PreferVector:
+          break;
+
+        case Qgis::RasterizedRenderingPolicy::ForceVector:
+          return false;
+      }
+
+      if ( !qgsDoubleNear( mLayerOpacity, 1.0 ) )
+        return true;
+    }
   }
 
   return false;
@@ -855,7 +868,7 @@ void QgsRasterLayerRenderer::drawElevationMap()
                gdalDsOutput.get(),
                QgsGdalUtils::gdalResamplingAlgorithm( dataProvider->zoomedInResamplingMethod() ), nullptr ) )
         {
-          elevationBlock.reset( rotatedElevationBlock.release() );
+          elevationBlock = std::move( rotatedElevationBlock );
         }
 
         topLeft = QPoint( left, top );
